==> java-client-8.5.1-sources.jar: io/appium/java_client/pagefactory/interceptors/InterceptorOfAListOfElements.java <==
/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.appium.java_client.pagefactory.interceptors;

import io.appium.java_client.proxy.MethodCallListener;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.pagefactory.ElementLocator;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

public abstract class InterceptorOfAListOfElements implements MethodCallListener {
    protected final ElementLocator locator;

    public InterceptorOfAListOfElements(ElementLocator locator) {
        this.locator = locator;
    }

    protected abstract Object getObject(
            List<WebElement> elements, Method method, Object[] args
    ) throws Throwable;

    @Override
    public Object call(Object obj, Method method, Object[] args, Callable<?> original) throws Throwable {
        if (Object.class.equals(method.getDeclaringClass())) {
            return original.call();
        }

        List<WebElement> realElements = new ArrayList<>(locator.findElements());
        return getObject(realElements, method, args);
    }
}

==> java-client-8.6.0-sources.jar: io/appium/java_client/pagefactory/interceptors/InterceptorOfAListOfElements.java <==
/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.appium.java_client.pagefactory.interceptors;

import io.appium.java_client.proxy.MethodCallListener;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.pagefactory.ElementLocator;

import javax.annotation.Nullable;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

public abstract class InterceptorOfAListOfElements implements MethodCallListener {
    protected final ElementLocator locator;

    public InterceptorOfAListOfElements(@Nullable ElementLocator locator) {
        this.locator = locator;
    }

    protected abstract Object getObject(
            List<WebElement> elements, Method method, Object[] args
    ) throws Throwable;

    @Override
    public Object call(Object obj, Method method, Object[] args, Callable<?> original) throws Throwable {
        if (locator == null || Object.class.equals(method.getDeclaringClass())) {
            return original.call();
        }

        List<WebElement> realElements = new ArrayList<>(locator.findElements());
        return getObject(realElements, method, args);
    }
}
